﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

using Kairai;
using Windows.Graphics.Display;
using Windows.UI.Core;

using Windows.ApplicationModel;
using Windows.Storage;
using Windows.Storage.Streams;

namespace TestApp
{
    public enum Action
    {
        Walk = 0,
        Stay,
        Kick,
        Stretch,
        Max
    };

    public enum Mesh
    {
        Rai = 0,
        Max
    };

    /// <summary>
    /// それ自体で使用できる空白ページまたはフレーム内に移動できる空白ページ。
    /// </summary>
    public sealed partial class MainPage : Page
    {
        bool m_windowVisible;
        KairaiApi m_kairai;
        bool m_loading;

        public MainPage()
        {
            this.InitializeComponent();

            m_kairai = new KairaiApi();

            DisplayInformation displayInf = DisplayInformation.GetForCurrentView();
            displayInf.DpiChanged += OnDpiChanged;
            displayInf.OrientationChanged += OnOrientationChanged;
            DisplayInformation.DisplayContentsInvalidated += OnDisplayContentsInvalidated;

            swapChainPanel.CompositionScaleChanged += OnCompositionScaleChanged;
            swapChainPanel.SizeChanged += OnSwapChainPanelSizeChanged;

            CoreWindow window = Window.Current.CoreWindow;
            window.VisibilityChanged += OnVisibilityChanged;
            window.PointerPressed += OnPointerPressed;
            window.PointerMoved += OnPointerMoved;
            window.PointerWheelChanged += OnPointerWheelChanged;
            m_kairai.Initialize(swapChainPanel);

            //Timer
            DispatcherTimer timer = new DispatcherTimer();
            timer.Tick += TimerTick;
            timer.Interval = new TimeSpan(0, 0, 1);
            timer.Start();
        }
        void OnDpiChanged(DisplayInformation sender, Object args)
        {
            m_kairai.OnDpiChanged(sender, args);
        }
        void OnOrientationChanged(DisplayInformation sender, Object args)
        {
            m_kairai.OnOrientationChanged(sender, args);
        }
        void OnDisplayContentsInvalidated(DisplayInformation sender, Object args)
        {
            m_kairai.OnDisplayContentsInvalidated(sender, args);
        }
        void OnCompositionScaleChanged(SwapChainPanel sender, Object args)
        {
            m_kairai.OnCompositionScaleChanged(sender, args);
        }
        void OnSwapChainPanelSizeChanged(Object sender, SizeChangedEventArgs e)
        {
            m_kairai.OnSwapChainPanelSizeChanged(sender, e);
        }
        public void LoadInternalState()
        {
            m_kairai.StartRenderLoop();
        }
        public void SaveInternalState()
        {
            m_kairai.StopRenderLoop();
        }
        void OnVisibilityChanged(CoreWindow sender, VisibilityChangedEventArgs args)
        {
            m_windowVisible = args.Visible;
            if (m_windowVisible)
            {
                m_kairai.StartRenderLoop();
            }
            else
            {
                m_kairai.StopRenderLoop();
            }
        }

        public void TimerTick(object sender, object e)
        {
            switch (m_kairai.GetKairaiState())
            {
                case KAIRAI_STATE.KAIRAI_INIT:
                    if (m_kairai.IsBusy())
                    {
                        break;
                    }
                    LoadMeshes();
                    break;
            }
        }

        public async void LoadMeshes()
        {
            if (m_loading)
            {
                return;
            }
            m_loading = true;

            String fileName = "Rai.xbone";
            StorageFolder localFolder = Package.Current.InstalledLocation;
            var loadFile = await localFolder.GetFileAsync(fileName);
            var randomStream = await loadFile.OpenAsync(FileAccessMode.Read);

            ulong buffSize = randomStream.Size;
            byte[] buffer = new byte[buffSize];

            DataReader reader = new DataReader(randomStream.GetInputStreamAt(0));
            await reader.LoadAsync((uint)buffSize);
            reader.ReadBytes(buffer);

            m_kairai.LoadBin(buffer);
            m_kairai.SetGame(true);

            MoveMesh(Mesh.Rai, 0, 0, 0);

            m_kairai.SetKairaiState(KAIRAI_STATE.KAIRAI_READY);
            m_kairai.PlayMesh((int)Mesh.Rai, (int)Action.Stay, true, false);
        }

        void MoveMesh(Mesh meshEnum, float x, float y, float z)
        {
            int meshNo = (int)meshEnum;
            MetroMesh mesh = m_kairai.GetMesh(meshNo);
            mesh.OffsetX = x;
            mesh.OffsetY = y;
            mesh.OffsetZ = z;
            m_kairai.SetMesh(meshNo, mesh);
        }

        void OnPointerWheelChanged(Object sender, PointerEventArgs e)
        {
            m_kairai.OnPointerWheelChanged(sender, e);
        }

        void OnPointerMoved(Object sender, PointerEventArgs e)
        {
            int rai = (int)Mesh.Rai;
            Point pos = e.CurrentPoint.Position;
            pos = m_kairai.GetDirection(rai, pos);
            float x = (float)pos.X;
            float z = (float)pos.Y;
            m_kairai.MoveMesh(rai, x, 0, z);
        }

        void OnPointerPressed(object sender, PointerEventArgs e)
        {
        }

        private void WalkButton_Click(object sender, RoutedEventArgs e)
        {
            m_kairai.PlayMesh((int)Mesh.Rai, (int)Action.Walk, true, true);
        }

        private void StayButton_Click(object sender, RoutedEventArgs e)
        {
            m_kairai.PlayMesh((int)Mesh.Rai, (int)Action.Stay, true, false);
        }

        private void KickButton_Click(object sender, RoutedEventArgs e)
        {
            m_kairai.PlayMesh((int)Mesh.Rai, (int)Action.Kick, false, false);
        }

        private void StretchButton_Click(object sender, RoutedEventArgs e)
        {
            m_kairai.PlayMesh((int)Mesh.Rai, (int)Action.Stretch, false, false);
        }
    }
}
